package shooting;

import java.awt.*;
import java.util.Random;
import javax.swing.ImageIcon;

/**
 * Canvas class 
 * Graphics display canvas(Extends Canvas)<p>
 * main loop here
*/
public class MyCanvas extends Canvas implements Runnable{
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	ObjectPool objectpool;
	KeyInput keyinput;
	Image imgBuf ;
	Graphics gBuf;
	Random random;
	Title title;
	Score score;	
	Image background; 

	/**
	 * Scene management field
	 * 0:Title Scene 1:Game main interface
	 */
	public static int scene;
	static final int SCENE_TITLE = 0;
	static final int SCENE_GAMEMAIN = 1;
	
	
	public static boolean gameover;
	int counter;
	
	
	//shoot key management
	//it is pressed
	static final int SHOT_PRESSED = 1;
	//it is pressed and released
	static final int SHOT_DOWN = 2;
	static int shotkey_state;


	/**
	 * 
	 */
	MyCanvas()
	{
		//keylistener implementation
		keyinput = new KeyInput();
		addKeyListener(keyinput);
		setFocusable(true);	
		random = new Random();			//random number
		title = new Title();
		score = new Score();
	}

	/**
	 * Initialization<p>
	 * Called when the application starts or restarts<br>
	 * Initializes the fields that are used in the game
	 */
	public void init()
	{
		//loading a background image from file
		ImageIcon icon = new ImageIcon(getClass().getResource("bg.jpg"));
	    background = icon.getImage();
		objectpool = new ObjectPool();
		Score.loadScore();
		
		//Starts from title scene
		scene = SCENE_TITLE; 
		gameover = false;
		//Initialize Level
		Level.initLevel();
		//Initialize Score
		Score.initScore();
	}
	
	/**
	 * Initialize the thread from outside
	 */
	public void initThread()
	{
		Thread thread = new Thread(this);
		thread.start();
	}
	
	/**
	 * paint method<p>
	 * called when repaint() is ran.
	 * Copies graphics from the off screen buffered and displays
	 * @param g
	 **/
	
	public void paint(Graphics g)
	{
		//Copy image from the off screen buffered
		g.drawImage(imgBuf, 0, 0, this);
	}

	/**
	 * main loop
	 */
	public void run()
	{
			
		//Create off screen buffered
		imgBuf = createImage(500, 500);
		gBuf = imgBuf.getGraphics();
		
		for(counter = 0; ; counter++)
		{
			shotkey_state = keyinput.checkShotKey();
			gBuf.drawImage(background, 0, 0, getParent());			

			//Scene switch
			switch (scene)
			{
				//Title
				case 0:
					title.drawTitle(gBuf);
					score.drawScore(gBuf);
					score.drawHiScore(gBuf);
					
					//When space is pressed
					if (shotkey_state == SHOT_DOWN)
					{
						//Go to main game screen
						scene = SCENE_GAMEMAIN;
						ObjectPool.player.hitpoints = 4-Game.difficulty;
					}
					break;
				
				//Game main screen
				case 1:
					gameMain();
					break;
			}

			//Run repaint
			repaint();
			
			try{
				Thread.sleep(16);				//loop wait
			}
			catch(InterruptedException e)
			{}
		}
	}

	/**
	 * Override update method to prevent repaint the whole screen
	 * @param g Graphic handler
	 */
	public void update(Graphics g)
	{
		paint(g);
	}
	
	//function loads different backgrounds based on the random number generator
	private void loadBackground() {
		int random = (int) ( 1 + Math.random()*7);
		switch (random)
		{
		case 1:
			 ImageIcon icon = new ImageIcon(getClass().getResource("bg1.jpg"));
		     background = icon.getImage();
			break;
		case 2:
			 ImageIcon icon2 = new ImageIcon(getClass().getResource("bg2.jpg"));
		     background = icon2.getImage();
			break;
		case 3:
			 ImageIcon icon3 = new ImageIcon(getClass().getResource("bg3.jpg"));
		     background = icon3.getImage();
			break;
		case 4:
			 ImageIcon icon4 = new ImageIcon(getClass().getResource("bg4.jpg"));
		     background = icon4.getImage();
			break;
		case 5:
			 ImageIcon icon5 = new ImageIcon(getClass().getResource("bg5.jpg"));
		     background = icon5.getImage();
			break;
		case 6:
			 ImageIcon icon6 = new ImageIcon(getClass().getResource("bg6.jpg"));
		     background = icon6.getImage();
			break;
		case 7:
			 ImageIcon icon7 = new ImageIcon(getClass().getResource("bg7.jpg"));
		     background = icon7.getImage();
			break;
		default:
		}
   }
	
	/**
	 * Game main
	 */
	void gameMain()
	{
		//Game over flag
		if (objectpool.isGameover())
		{
			//Display Game over
			
			title.drawGameover(gBuf);
			
			
			if (shotkey_state == SHOT_DOWN)
			{
				//Save score to high score if its necessary
				Score.compareScore();
				try {
					Database.Execute();					
					
				} catch (Exception e) {
					e.printStackTrace();
				}

				//Initialize the game
				init();
			}			
		}	
		
		//Collision manager
		objectpool.getColision();
		objectpool.movePlayer(keyinput);
		
		//Enemy production frequenct
		//Increases as level increased
		if (counter % (100 - Level.getLevel() * 10) == 0)
		{
			ObjectPool.newEnemy(100 + random.nextInt(300), 0);
		}

		
		//A function that increases the level based on the number of points earned
		if ((Score.myscore > Level.getLevel()*300) && Level.getLevel() < 8 )
		{
			Level.addLevel();
			loadBackground();	
		}
		
		//Player shoots bullet with specific frequency
		//When space key is pressed and if the counter is a product of 3
		if ((shotkey_state == SHOT_PRESSED)&&(counter % 3 == 0))
		{
			objectpool.shotPlayer();
			objectpool.shotSidekick();
		}
		

		//All game object draw process
		objectpool.drawAll(gBuf);
		//Draw score
		score.drawScore(gBuf);
		score.drawHiScore(gBuf);
		
	}
}



