package shooting;
import java.awt.*;

/**
 * Game object management class
 * Holds the instances of player, bullet, enemy classed
 * also manages interaction of objects(e.g. collision)
*/
public class ObjectPool
{
	
	/**
	 * Array to store enemy objects
	 */	
	static Enemy[] enemy;
	/**
	 * Array to store enemy bullets objects
	 */
	static Bullet[] bullet;
	/**
	 * Array to store player's bullets
	 */
	static MyBullet[] mybullet;
	/**
	 * Array to store explosion particle objects
	 */
	static Particle[] particle;
	
	/**
	 * Array to store Nuclear Graphic objects
	 */
	static NukeGraphic[] nukegraphic;
	
	/**
	 * Array to store booster objects
	 */	
	static Booster[] booster;

	/**
	 * Player object instance
	 */
	static Player player;
	
	/**
	 * Sidekick object instance
	 */
	static Sidekick[] sidekick;
	
	//Constants
	static final int DIST_PLAYER_TO_BULLET = 20;
	static final int DIST_PLAYER_TO_ENEMY = 20;
	static final int DIST_ENEMY_TO_MYBULLET = 20;
	
	//Max Constants
	static final int BULLET_MAX = 1000;
	static final int ENEMY_MAX = 7;
	static final int SIDEKICK_MAX = 2;
	static final int PARTICLE_MAX = 100;
	static final int NUKEGRAPHIC_MAX = 360;
	static final int MYBULLET_MAX = 8;
	static final int BOOSTER_MAX = 5;	
	
	static int k = 20;
    
	/**
	 * Constructor
	 */
	ObjectPool()
	{
		//Create player object
		player = new Player(3, 250, 400, 4);
		player.active = true;
		
		sidekick = new Sidekick[SIDEKICK_MAX];
		for(int i = 0; i < sidekick.length; i++)
		{
				sidekick[i] = new Sidekick(player);
		}

		
		//get array for storing enemy bullets objects
		bullet = new Bullet[BULLET_MAX];
		for(int i = 0; i < bullet.length; i++)
		{
				bullet[i] = new Bullet();
		}
		
		//get array for storing enemy objects
		enemy = new Enemy[ENEMY_MAX];
		for(int i = 0; i < enemy.length; i++)
		{
				enemy[i] = new Enemy(player);
		}
		
		//get array for storing player's bullet objects
		mybullet = new MyBullet[MYBULLET_MAX];
		for(int i = 0; i < mybullet.length; i++)
		{
				mybullet[i] = new MyBullet();
		}

		//get array for storing explosion particle objects
		particle = new Particle[PARTICLE_MAX];
		for(int i = 0; i < particle.length; i++)
		{
				particle[i] = new Particle();
		}
		//get array for storing nuclear graphic objects
		nukegraphic = new NukeGraphic[NUKEGRAPHIC_MAX];
		for(int i = 0; i < nukegraphic.length; i++)
		{
				nukegraphic[i] = new NukeGraphic();
		}
		//get array for storing boosters
		booster = new Booster[BOOSTER_MAX];
		for(int i = 0; i < booster.length; i++)
		{
				booster[i] = new Booster();
		}
	}

	/**
	 * Graphics management
	 * Draw all objects on screen
	 */
	public void drawAll(Graphics g)
	{
        doGameObjects(g, bullet);
        doGameObjects(g, enemy);
        doGameObjects(g, mybullet);
        doGameObjects(g, particle);
        doGameObjects(g, nukegraphic);
        doGameObjects(g, booster);
        doGameObjects(g, sidekick);
		player.draw(g);
	}

	/**
	 * Draw only instances that are activated
	 * referring to each array
	 */
    public void doGameObjects(Graphics g, GameObject[] objary)
    {
        for (int i = 0; i < objary.length; i++) {
			
            if (objary[i].active == true)
			{
                objary[i].move();
                objary[i].draw(g);				
            }
        }
    }
    
    public static void sidekick()
    {
    	if (Sidekick.isactive == false)
    	{
    		newSidekick(player.x+30, player.y);      
    		Sidekick.isactive = true;
    	}
    	else 
    	{
    		newSidekick(player.x-30, player.y);
    		Sidekick.is2active = true;
    	}
    		
    }
    //This function executes the implementation of nuclear charge, damaging all enemies on screen
    public static void atomic()
	{
    	
    	for (int i = 0; i < 360; i+=5)  // i linked to direction
		{
			ObjectPool.newNukegraphic(250, 220, i, 10);
			// (x coord, y coord, idirection, speed, type)
		}
    	for (int i = 0; i < bullet.length; i++)
    		bullet[i].active = false;
    	
    	for (int i = 0; i < enemy.length; i++) { //checks for available enemies in the array
    		if ( enemy[i].active == true ) //makes sure that the enemy is activated
    		{    			
    			while (enemy[i].hitpoint >= 0)//destroys the enemies on screen, producing video effects
    			{
    				enemy[i].hit();
    				newNukegraphic(enemy[i].x, enemy[i].y, 180+40*k, 3);    				
    				k++;
    			}
    		}
		  }			
	}
 
    /**
     * Create Bullet object, initialize or reuse
     * @param ix x coordinate
     * @param iy y coordinate
     * @param idirection direction
     * @param ispeed speed
     * @param type 
     * @return Bullet ID  ireturns -1 when there is no space 
     */
	public static int newBullet(double ix, double iy, double idirection, double ispeed, int type)
	{
		for (int i = 0; i < BULLET_MAX; i++)
		{
			if ((bullet[i].active) == false)
			{
				bullet[i].activate(ix, iy, idirection, ispeed, type);
				return i;
			}
		}
		return -1;		//not found
	}
	
	
	 /**
     * Create Booster object, initialize or reuse
     * @param ix x coordinate
     * @param iy y coordinate
     * @return Booster ID i returns -1 when there is no space 
     */
	public static int newBooster(double ix, double iy)
	{
		for (int i = 0; i < BOOSTER_MAX; i++)
		{
			if ((booster[i].active) == false)
			{
				booster[i].activate(ix, iy);
				return i;
			}
		}
		return -1;		//not found
	}

    /**
     * Create Enemy object, initialize or reuse
     * @param ix x coordinate
     * @param iy y coordinate
     * @return Enemy ID i returns -1 when there is no space
     */
	public static int newEnemy(double ix, double iy)
	{
		for (int i = 0; i < ENEMY_MAX; i++)
		{
			if ((enemy[i].active) == false)
			{
				enemy[i].activate(ix, iy);
				return i;
			}
		}
		return -1;		//not found
	}
	
	
	 /**
     * Create Enemy object, initialize or reuse
     * @param ix x coordinate
     * @param iy y coordinate
     * @return Enemy ID i returns -1 when there is no space
     */
	public static int newSidekick(double ix, double iy)
	{
		for (int i = 0; i < SIDEKICK_MAX; i++)
		{
			if ((sidekick[i].active) == false)
			{
				sidekick[i].activate(ix, iy);
				return i;
			}
		}
		return -1;		//not found
	}

    /**
     * Create Player Bullet object, initialize or reuse
     * @param ix x coordinate
     * @param iy y coordinate
	 * @return Player Bullet ID (returns negative one when there is no space)
     */
	public static int newMyBullets(double ix, double iy)
	{
		for (int i = 0; i < MYBULLET_MAX; i++)
		{
			if ((mybullet[i].active) == false)
			{
				mybullet[i].activate(ix, iy);
				if (MyBullet.type == 1) //checks if the bullets are boosted
					MyBullet.counter--; //decreases boost ammo
				if (MyBullet.counter < 0) //checks if the player has any booster ammo left
					MyBullet.type = 0; //switches back to regular ammo
				return i;
			}
				
		}
		return -1;		//not found
	}

    /**
     * Create Explosion particle object, initialize or reuse
     * @param ix x coordinate
     * @param iy y coordinate
     * @param idirection direction
     * @param ispeed speed
     * @return Explosion particle ID i returns -1 when there is no space
     */
	public static int newParticle(double ix, double iy, double idirection, double ispeed)
	{
		for (int i = 0; i < PARTICLE_MAX; i++)
		{
			if ((particle[i].active) == false)
			{
				particle[i].activate(ix, iy, idirection, ispeed);
				return i;
			}
		}
		return -1;		//not found
	}

	  /**
     * Create Nuclear Graphic object, initialize or reuse
     * @param ix x coordinate
     * @param iy y coordinate
     * @param idirection direction
     * @param ispeed speed
     * @return Nuclear Graphic ID i returns -1 when there is no space
     */
	public static int newNukegraphic(double ix, double iy, double idirection, double ispeed)
	{
		for (int i = 0; i < NUKEGRAPHIC_MAX; i++)
		{
			if ((nukegraphic[i].active) == false)
			{
				nukegraphic[i].activate(ix, iy, idirection, ispeed);
				return i;
			}
		}
		return -1;		//not found
	}
	
	/**
	 * Player shooting
	 */
	public void shotPlayer()
	{
		//Only available when player object is active
		if (player.active)
		{
			newMyBullets(player.x, player.y);
		}
	}
	
	public void shotSidekick()
	{
		//Only available when player object is active
		if (sidekick[0].active)
		{
			newMyBullets(player.x+30, player.y);
		}
		if (sidekick[1].active)
			newMyBullets(player.x-30, player.y);
	}

	/**
	 * Player movement management
	 * @param keyinput key input instance
	 */
	public void movePlayer(KeyInput keyinput)
	{
		player.move(keyinput.getXDirection(), keyinput.getYDirection());
	}
	
	/**
	 * Returns distance between two objects
	 * @param ga game object a
	 * @param gb game object b
	 * @return distance
	 */
	public double getDistance(GameObject ga, GameObject gb)
	{
		//Pythagorean theorem
		double Xdiff = Math.abs(ga.x - gb.x);
		double Ydiff = Math.abs(ga.y - gb.y);
		return Math.sqrt(Math.pow(Xdiff,2) + Math.pow(Ydiff,2));
	}

	/**
	 * collision judgment
	 */
	public void getColision()
	{
		//enemy bullets and player
        for (int i = 0; i < bullet.length; i++) {
			if ((bullet[i].active)&&(player.active))
			{
				//Judgment
				if (getDistance(player, bullet[i]) < DIST_PLAYER_TO_BULLET)
				{					
					if (sidekick[0].active == true && sidekick[1].active == true)
						sidekick[1].hit();
					else if (sidekick[0].active == true)
						sidekick[0].hit();					
					else if (player.hitpoints > 0) //Player loses a life
						{
							player.hitpoints--;
							newParticle(player.x, player.y, bullet[i].x, 3);
							
						}
					else //player dies
					{
						player.active = false;
						
						//Create explosion
						for (int j = 0; j < 360; j += 20)
						{
							newParticle(player.x, player.y, j, 2);
						}
					}
					//Bullet disappears
					bullet[i].active = false;
											
				}	
			}
        }

		//Player bullet and enemy
        for (int i = 0; i < enemy.length; i++) {
			if (enemy[i].active == true)
			{
				for (int j = 0; j < mybullet.length; j++)
				{
					if (mybullet[j].active == true)
					{
						//judgment
						if (getDistance(enemy[i], mybullet[j]) < DIST_ENEMY_TO_MYBULLET)
						{
							newParticle(mybullet[j].x, mybullet[j].y, 270, 2);
							//Decrease enemy hit point
							if (MyBullet.type == 0) //checks the type of bullets
								enemy[i].hit(); //regular bullets take away one hitpoint
							else //booster bullets take away 3 hitpoints
							{
								enemy[i].hit();
								if (enemy[i].hitpoint >= 0) //making sure that the enemy has hitpoints left
									enemy[i].hit();
								if (enemy[i].hitpoint >= 0)
									enemy[i].hit();
							}
							//Bullet disappears
							mybullet[j].active = false;
						}
					}
				}
			}
        }

		//Enemy and player
        for (int i = 0; i < enemy.length; i++) {
			if ((enemy[i].active)&&(player.active))
			{
				//judgment
				if (getDistance(player, enemy[i]) < DIST_PLAYER_TO_ENEMY)
				{
					
					if (player.hitpoints > 0) //player loses hitpoints
					{
						player.hitpoints--;
						newParticle(player.x, player.y, enemy[i].x, 2);
					}
					else //player disappears
					{						
						player.active = false;
	
						//Create explosion
						for (int j = 0; j < 360; j += 20)
						{
							newParticle(player.x, player.y, j, 2);
						}
						//Enemy does not disappear
					}
				}
			}
        }
        
        //booster and player
        for (int i = 0; i < booster.length; i++) {
			if ((booster[i].active)&&(player.active))
			{
				//judgment
				if (getDistance(player, booster[i]) < DIST_PLAYER_TO_ENEMY)
				{
					Score.addScore(5);
					//player gets a hitpoint, booster disappears
					if (booster[i].type == 1)
						player.hitpoints++;
					else if (booster[i].type == 0) //player gets 30 rounds of boosted ammo
					{
						MyBullet.counter += 30;	
						MyBullet.type = 1;
					}
					else //player gets a nuclear charge
					{
						Booster.atomcounter++;
					}
				booster[i].active = false; //deactivating the used booster
				
				}
			}
        }
	}
		
	/**
	 * Game over 
	 * @return returns true when game over
	 */
	public boolean isGameover()
	{
		return !player.active;
	}
}

